<?php

include_once(dirname(__FILE__) . '/TeamBOTAPI.class.php');

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class TeamBOT {

    private static $instance;

    public function __construct() {}

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        
        return self::$instance;
    }

    public function init() {
        add_action( 'admin_menu', [$this, 'menu'] );
        add_action( 'admin_init', [$this, 'settings'] );
        add_action( 'admin_enqueue_scripts', [$this, 'admin_scripts'] );
        add_action( 'wp_enqueue_scripts', [$this, 'scripts']);
        add_action( 'rest_api_init', [$this, 'register_apis'] );
        add_action( 'init', [$this, 'register'] );
        add_action( 'add_meta_boxes', [$this, 'metaboxes']);
        add_action( 'save_post', [$this, 'save_metabox']);
        add_action( 'admin_notices', [$this, 'admin_notices']);

        add_action('admin_post_teambot_clear_cache', [$this, 'clear_cache']);
        add_action('admin_post_nopriv_teambot_clear_cache', [$this, 'clear_cache']);
        add_action('wp_ajax_get_teambot_tasks_lists', [$this, 'get_lists']);
        add_action('wp_ajax_nopriv_get_teambot_tasks_lists', [$this, 'get_lists']);
        add_action('wp_ajax_teambot_form_submit', [$this, 'teambot_form_submit']);
        add_action('wp_ajax_nopriv_teambot_form_submit', [$this, 'teambot_form_submit']);

        add_filter('manage_teambot-roadmap_posts_columns', [$this, 'shortcode_column']);
        add_action('manage_teambot-roadmap_posts_custom_column', [$this, 'roadmap_column_content'], 10, 2);
        add_filter('manage_teambot-form_posts_columns', [$this, 'shortcode_column']);
        add_action('manage_teambot-form_posts_custom_column', [$this, 'form_column_content'], 10, 2);

        add_shortcode( 'teambot-form', [$this, 'teambot_form'] );
        add_shortcode( 'teambot-roadmap', [$this, 'teambot_roadmap'] );

    }

    public function admin_notices() {
        if (isset($_SESSION['teambot_success_message'])) {
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php echo esc_html($_SESSION['teambot_success_message']); ?></p>
            </div>
            <?php
            // Clear the session variable after displaying the notice
            unset($_SESSION['teambot_success_message']);
        }
    }

    public function menu() {
        add_menu_page( 
            __( 'TeamBOT\'s settings', 'TeamBOT' ),
            'TeamBOT',
            'manage_options',
            'teambot-settings',
            [$this, 'settings_page'],
            plugins_url( 'teambot/assets/img/logo-20.png' ),
            6
        ); 

        add_submenu_page(
            'teambot-settings', 
            'Roadmaps', 
            'Roadmaps', 
            'manage_options',
            'edit.php?post_type=teambot-roadmap',
        );

        add_submenu_page(
            'teambot-settings', 
            'Forms', 
            'Forms', 
            'manage_options',
            'edit.php?post_type=teambot-form', 
        );
    }
    
    public function settings_page(){
        include_once(dirname(__FILE__) . '/templates/settings.php');
    }

    public function settings(){
        register_setting( 'teambot-settings', 'teambot-apikey' );
    }

    public function admin_scripts() {
        if (is_admin()) {
            wp_enqueue_style( 'teambot-admin-css', plugins_url('teambot/assets/css/admin-style.css'), false, '1.0.0' );
            
            // Enqueue jQuery
            wp_enqueue_script('jquery');

            // Enqueue your custom admin script after jQuery
            wp_enqueue_script('teambot-admin-script', plugins_url('teambot/assets/js/admin-script.js'), array('jquery'), '1.0', true);
            
            // Pass the AJAX URL to the JavaScript file
            wp_localize_script('teambot-admin-script', 'admin_ajax', array('ajax_url' => admin_url('admin-ajax.php')));
        }
    }

    public function scripts() {

        wp_enqueue_style('teambot-style', plugins_url('assets/css/style.css', __FILE__), false, '1.0.0');
        
        // Enqueue jQuery
        wp_enqueue_script('jquery');

        // Enqueue your custom admin script after jQuery
        wp_enqueue_script('teambot-script', plugins_url('teambot/assets/js/script.js'), array('jquery'), '1.0', true);
        
        // Pass the AJAX URL to the JavaScript file
        wp_localize_script('teambot-script', 'admin_ajax', array('ajax_url' => admin_url('admin-ajax.php')));
        
    }

    public function register() {

        if ( ! session_id() ) {
            session_start();
        }

        $labelsRoadmap = array(
            'name'               => 'Roadmaps',
            'singular_name'      => 'Raodmap',
            'menu_name'          => 'Roadmaps',
            'name_admin_bar'     => 'Roadmap',
            'add_new'            => 'Add New',
            'add_new_item'       => 'Add New Roadmap',
            'new_item'           => 'New Roadmap',
            'edit_item'          => 'Edit Roadmap',
            'view_item'          => 'View Roadmap',
            'all_items'          => 'All Roadmaps',
            'search_items'       => 'Search Roadmaps',
            'parent_item_colon'  => 'Parent Roadmaps:',
            'not_found'          => 'No roadmaps found.',
            'not_found_in_trash' => 'No roadmaps found in Trash.',
        );
    
        $argsRoadmap = array(
            'labels'              => $labelsRoadmap,
            'public'              => true,
            'publicly_queryable'  => true,
            'show_ui'             => true,
            'show_in_menu'        => false,
            'show_in_rest'        => true,
            'query_var'           => true,
            'rewrite'             => array( 'slug' => 'teambot-roadmap' ),
            'capability_type'     => 'post',
            'has_archive'         => true,
            'hierarchical'        => false,
            'menu_position'       => null,
            'taxonomies'          => array(),
            'supports'            => array( 'title' )
        );
    
        register_post_type( 'teambot-roadmap', $argsRoadmap );

        $labelsForm = array(
            'name'               => 'Forms',
            'singular_name'      => 'Form',
            'menu_name'          => 'Forms',
            'name_admin_bar'     => 'Form',
            'add_new'            => 'Add New',
            'add_new_item'       => 'Add New Form',
            'new_item'           => 'New Form',
            'edit_item'          => 'Edit Form',
            'view_item'          => 'View Form',
            'all_items'          => 'All Forms',
            'search_items'       => 'Search Forms',
            'parent_item_colon'  => 'Parent Forms:',
            'not_found'          => 'No forms found.',
            'not_found_in_trash' => 'No forms found in Trash.',
        );
    
        $argsForm = array(
            'labels'              => $labelsForm,
            'public'              => true,
            'publicly_queryable'  => true,
            'show_ui'             => true,
            'show_in_menu'        => false,
            'show_in_rest'        => true,
            'query_var'           => true,
            'rewrite'             => array( 'slug' => 'teambot-form' ),
            'capability_type'     => 'post',
            'has_archive'         => true,
            'hierarchical'        => false,
            'menu_position'       => null,
            'taxonomies'          => array(),
            'supports'            => array( 'title' ),
        );
    
        register_post_type( 'teambot-form', $argsForm );
    }

    public function metaboxes() {
        add_meta_box(
            'teambot_roadmap_meta_box', 
            'TeamBOT roadmap meta box', 
            [$this, 'metabox_callback'],
            'teambot-roadmap', 
            'normal', 
            'default' 
        );

        add_meta_box(
            'teambot_form_meta_box', 
            'TeamBOT form meta box', 
            [$this, 'metabox_callback'],
            'teambot-form', 
            'normal', 
            'default' 
        );
    }

    public function metabox_callback($post) {

        $teamBOTAPI = TeamBOTAPI::getInstance();

        if($post->post_type == 'teambot-roadmap'){

            $projects = $teamBOTAPI->getProjects();

            include_once(dirname(__FILE__) . '/templates/metaboxes/teambot-roadmap.php');

        }



        if($post->post_type == 'teambot-form'){

            $projects   = $teamBOTAPI->getProjects();
            $users      = $teamBOTAPI->getUsers();

            include_once(dirname(__FILE__) . '/templates/metaboxes/teambot-form.php');

        }
        
    }

    public function save_metabox($post_id) {

        $post = get_post($post_id);

        if($post->post_type == 'teambot-roadmap'){
            // Check if the nonce is set
            if (!isset($_POST['teambot_roadmap_meta_box_nonce'])) {
                return;
            }

            // Verify the nonce
            if (!wp_verify_nonce($_POST['teambot_roadmap_meta_box_nonce'], 'teambot_roadmap_meta_box')) {
                return;
            }

            // Check if this is an autosave
            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                return;
            }

            // Check the user's permissions
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }

            // Save the select input field value
            if (isset($_POST['teambot-project'])) {
                update_post_meta($post_id, '_teambot-project', sanitize_text_field($_POST['teambot-project']));
            }
        }

        if($post->post_type == 'teambot-form'){
            // Check if the nonce is set
            if (!isset($_POST['teambot_form_meta_box_nonce'])) {
                return;
            }

            // Verify the nonce
            if (!wp_verify_nonce($_POST['teambot_form_meta_box_nonce'], 'teambot_form_meta_box')) {
                return;
            }

            // Check if this is an autosave
            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                return;
            }

            // Check the user's permissions
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }

            // Save the select input field value
            if (isset($_POST['teambot-project'])) {
                update_post_meta($post_id, '_teambot-project', sanitize_text_field($_POST['teambot-project']));
            }

            if (isset($_POST['teambot-user'])) {
                update_post_meta($post_id, '_teambot-user', sanitize_text_field($_POST['teambot-user']));
            }

            if (isset($_POST['teambot-list'])) {
                update_post_meta($post_id, '_teambot-list', sanitize_text_field($_POST['teambot-list']));
            }
        }
        
    }

    public function get_lists() {

        $teamBOTAPI = TeamBOTAPI::getInstance();

        wp_send_json_success($teamBOTAPI->getLists($_REQUEST['project']));
    }

    public function register_apis() {

        register_rest_route( 'teambot/v1', '/roadmap/(?P<id>\d+)', array(
          'methods' => 'GET',
          'callback' => [$this, 'teambot_roadmap_api'],
        ) );
    }

    public function clear_cache() {
        
        if (get_option('teambot-projects')) {
            delete_option('teambot-projects');
        }

        if (get_option('teambot-users')) {
            delete_option('teambot-users');
        }

        global $wpdb;

        // Get all options from the options table
        $options = $wpdb->get_results("SELECT option_name FROM {$wpdb->prefix}options 
        WHERE option_name LIKE 'teambot-lists-%' OR option_name LIKE 'teambot-roadmap-%'");

        foreach ($options as $option) {
            delete_option($option->option_name);
        }

        $_SESSION['teambot_success_message'] = 'Cache refreshed!';

        wp_safe_redirect(wp_get_referer());
        exit;
    }

    public function shortcode_column($columns) {
        $columns['shortcode'] = 'Shortcode';
        return $columns;
    }

    public function roadmap_column_content($column_name, $post_id) {
        if ($column_name === 'shortcode') {
            echo esc_html('[teambot-roadmap id="' . $post_id . '"]');
        }
    }

    public function form_column_content($column_name, $post_id) {
        if ($column_name === 'shortcode') {
            echo esc_html('[teambot-form id="' . $post_id . '"]');
        }
    }

    public function teambot_form($atts) {
        if(isset($atts['id'])){
            ob_start();
            include_once(dirname(__FILE__) . '/templates/shortcodes/form.php');
            $shortcode = ob_get_clean();

            return $shortcode;
        }else{
            return;
        }
    }

    public function teambot_form_submit() {
        // Get form data
        $id = isset($_POST['id']) ? sanitize_text_field($_POST['id']) : '';
        $subject = isset($_POST['subject']) ? sanitize_text_field($_POST['subject']) : '';
        $description = isset($_POST['description']) ? sanitize_textarea_field($_POST['description']) : '';
        $priority = isset($_POST['priority']) ? sanitize_text_field($_POST['priority']) : null;

        if(empty($id)) {
            status_header(400);
            wp_send_json_error('Form id is missing.');
        }

        // Validate form data
        if (empty($subject) || empty($description)) {
            status_header(400);
            wp_send_json_error('Subject and description are required.');
        }

        // Validate form data
        if (!empty($priority) && !in_array($priority, ['low', 'high', 'medium'])) {
            status_header(400);
            wp_send_json_error('Incorrect priority value.');
        }

        $post = get_post($id);

        if ($post) {
            
            if($post->post_type != 'teambot-form'){
                status_header(400);
                wp_send_json_error('Form id is invalid.');
            }

            // Get post meta data
            $post_meta = get_post_meta($id);
            
            $teambotUser = null;
            if(isset($post_meta['_teambot-user'])){
                $teambotUser = $post_meta['_teambot-user'][0];
            }
            
            $teambotList = null;
            if(isset($post_meta['_teambot-list'])){
                $teambotList = $post_meta['_teambot-list'][0];
            }

            $teamBOTAPI = TeamBOTAPI::getInstance();

            $task = $teamBOTAPI->addTask($teambotList, $subject, $description, $priority, $teambotUser);

            if($task === false){
                status_header(500);
                wp_send_json_error('Couldn\'t create the new register.');
            }
            
            // Send success response
            wp_send_json_success('Form submitted successfully!');
        } else {
            status_header(400);
            wp_send_json_error('Form id is invalid.');
        }

    }

    public function teambot_roadmap($atts) {
        if(isset($atts['id'])){

            $post = get_post($atts['id']);

            if ($post) {

                if($post->post_type != 'teambot-roadmap'){
                    return;
                }

                $post_meta = get_post_meta($atts['id']);

                $teambotProject = null;
                if(isset($post_meta['_teambot-project'])){
                    $teambotProject = $post_meta['_teambot-project'][0];
                }

                if(is_null($teambotProject)){
                    return;
                }

                $teamBOTAPI = TeamBOTAPI::getInstance();

                $roadmap = $teamBOTAPI->getRoadmap($teambotProject);
                
                if(empty($roadmap)){
                    return;
                }

                ob_start();
                include_once(dirname(__FILE__) . '/templates/shortcodes/roadmap.php');
                $shortcode = ob_get_clean();

                return $shortcode;
            } else {
                return;
            }
        }else{
            return;
        }
    }
    
    public function teambot_roadmap_api($atts) {
        
        if(isset($atts['id'])){

            $post = get_post($atts['id']);

            if ($post) {

                if($post->post_type != 'teambot-roadmap'){
                    return new WP_Error('invalid_id', 'Invalid ID.', array('status' => 400));
                }

                $post_meta = get_post_meta($atts['id']);

                $teambotProject = null;
                if(isset($post_meta['_teambot-project'])){
                    $teambotProject = $post_meta['_teambot-project'][0];
                }

                if(is_null($teambotProject)){
                    return new WP_Error('missing_configuration', 'Missing configuration.', array('status' => 400));
                }

                $teamBOTAPI = TeamBOTAPI::getInstance();

                $roadmap = $teamBOTAPI->getRoadmap($teambotProject);
                
                if(empty($roadmap)){
                    return new WP_Error('connection_error', 'TeamBOT connection error.', array('status' => 500));
                }

                return new WP_REST_Response([
                    'success' => true,
                    'data' => $roadmap]);
            } else {
                return new WP_Error('invalid_id', 'Invalid ID.', array('status' => 400));
            }
        }else{
            return new WP_Error('invalid_id', 'Invalid ID.', array('status' => 400));
        }
    }
}