<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class TeamBOTAPI {

    private $apiBaseUrl = 'https://teambot.ymtech.com.au/api/';

    private static $instance;

    public function __construct() {}

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        
        return self::$instance;
    }

    public function apiRequest($method='GET', $endpoint='teamwork/me', $params=null) {
        
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->apiBaseUrl . $endpoint);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        if($method == 'POST') {
            curl_setopt($ch, CURLOPT_POST, 1);
        }

        if(!is_null($params)) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        }

        $headers = array();
        $headers[] = 'Accept: application/json';
        $headers[] = 'Content-Type: application/json';

        $apiKey = get_option('teambot-apikey');

        if($apiKey) {
            $headers[] = 'Authorization: Bearer ' . $apiKey;
        }

        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        
        if (curl_errno($ch)) {
            return [
                'code'      => curl_errno($ch),
                'response'  => [
                    'message'   => 'Couldn\'t connect with TeamBOT server.'
                ]
            ];
        }

        curl_close($ch);

        return [
            'code'      => $httpcode,
            'response'  => json_decode($result, true)
        ];
    }

    public function getRoadmap($project){

        $roadmapCache = get_option('teambot-roadmap-' . $project);

        if(!$roadmapCache || $roadmapCache['expire'] < time() ){
            $roadmap =  $this->apiRequest('GET', 'teamwork/project/' . $project . '/roadmap');

            if(isset($roadmap['response']) && isset($roadmap['response']['roadmap'])){

                update_option('teambot-roadmap-' . $project, [
                    'expire'    => time() + 60 * 60 * 24,
                    'roadmap'  => $roadmap['response']['roadmap']
                ]);

                return $roadmap['response']['roadmap'];
            }else{
                return [];
            }

        } else {
            return $roadmapCache['roadmap'];
        }
    }

    public function getProjects() {

        $projectsCache = get_option('teambot-projects');

        if(!$projectsCache || $projectsCache['expire'] < time() ){
            $projects = $this->apiRequest('GET', 'teamwork/projects');

            if(isset($projects['response']) && isset($projects['response']['projects'])){

                update_option('teambot-projects', [
                    'expire'    => time() + 60 * 60 * 24,
                    'projects'  => $projects['response']['projects']
                ]);

                return $projects['response']['projects'];
            }else{
                return [];
            }

        } else {
            return $projectsCache['projects'];
        }
        
    }

    public function getUsers() {

        $usersCache = get_option('teambot-users');

        if(!$usersCache || $usersCache['expire'] < time() ){
            $users = $this->apiRequest('GET', 'teamwork/users');

            if(isset($users['response']) && isset($users['response']['users'])){

                update_option('teambot-users', [
                    'expire'    => time() + 60 * 60 * 24,
                    'users'  => $users['response']['users']
                ]);

                return $users['response']['users'];
            }else{
                return [];
            }

        } else {
            return $usersCache['users'];
        }
        
    }

    public function getLists($project) {

        $listsCache = get_option('teambot-lists-' . $project);

        if(!$listsCache || $listsCache['expire'] < time() ){
            $lists = $this->apiRequest('GET', 'teamwork/project/' . $project . '/tasklists');

            if(isset($lists['response']) && isset($lists['response']['tasklists'])){

                update_option('teambot-lists-' . $project, [
                    'expire'    => time() + 60 * 60 * 24,
                    'tasklists'  => $lists['response']['tasklists']
                ]);

                return $lists['response']['tasklists'];
            }else{
                return [];
            }

        } else {
            return $listsCache['tasklists'];
        }
        
    }

    public function addTask($list, $title, $description, $priority = null, $user = null) {
        $params = [
            'listId'        => intval($list),
            'title'         => $title,
            'description'   => $description
        ];

        if(!is_null($priority)){
            $params['priority'] = $priority;
        }

        if(!is_null($user)){
            $params['assignee'] = intval($user);
        }

        $task = $this->apiRequest('POST', 'teamwork/task', $params);

        if(isset($task['response']) && isset($task['response']['task'])){
            return $task['response']['task'];
        }else{
            return false;
        }
    }
        
}