<?php 
$apiKey = get_option('teambot-apikey');
$error  = "";
$me     = [];

if($apiKey){
    $teamBOTAPI = TeamBOTAPI::getInstance();
    $response = $teamBOTAPI->apiRequest('GET', 'teamwork/me');
    if($response['code'] != 200){

        if(isset($response['response']['message'])){
            $error = $response['response']['message'];
        }else{
            $error = 'Error while making API request to TeamBOT';
        }
        
    } else {
        $me = $response['response']['me'];
    }
}

?>
<div class="wrap">
    <h1>TeamBOT's settings</h1>

    <div class="card">
        <form method="post" action="options.php">
            <?php settings_fields('teambot-settings'); ?>
            <?php do_settings_sections('teambot-settings'); ?>

            <?php if(!$apiKey || $error): ?>
                <table class="form-table teambot-settings">
                    <tr valign="top">
                        <td scope="row"><img src="<?php echo plugins_url( 'teambot/assets/img/logo-20.png' ); ?>" /> &nbsp;<strong>API Key</strong></td>
                    </tr>
                    <tr valign="top">
                        <td scope="row"><p>Get your API key from your TeamBOT's account and complete the setup.</p></td>
                    </tr>
                    <tr valign="top">
                        <td scope="row"><input type="text" name="teambot-apikey" value="<?php echo esc_attr($apiKey); ?>" /></td>
                    </tr>

                    <?php if($error): ?>
                    <tr valign="top">
                        <td scope="row"><p class="error"><?php echo $error; ?></p></td>
                    </tr>
                    <?php endif; ?>

                    <tr valign="top">
                        <td scope="row"><?php submit_button(); ?></td>
                    </tr>
                </table>
            <?php else: ?>
                <table class="form-table teambot-settings">
                <tr valign="top">
                    <td scope="row" colspan="2"><img src="<?php echo plugins_url( 'teambot/assets/img/logo-20.png' ); ?>" /> &nbsp;<strong>Linked account information</strong></td>
                </tr>

                <tr valign="top">
                    <td>Name:</td>
                    <td><?php echo $me['lastname'] . ', ' . $me['firstname'];?></td>
                </tr>

                <tr valign="top">
                    <td>Email:</td>
                    <td><?php echo $me['email'];?></td>
                </tr>

                <tr valign="top">
                    <td>Company:</td>
                    <td><?php echo $me['company'];?></td>
                </tr>

                <input type="hidden" name="teambot-apikey" value="" />

                <tr valign="top">
                    <td scope="row" colspan="2"><?php submit_button('Revoke access', 'danger'); ?></td>
                </tr>
            </table>
            <?php endif; ?>
        </form>

    </div>

    <?php if($apiKey && !$error): ?>
        <div class="card">

            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                <input type="hidden" name="action" value="teambot_clear_cache">
                <table class="form-table teambot-settings">
                    <tr valign="top">
                        <td scope="row"><strong>API Cache</strong></td>
                    </tr>
                    <tr valign="top">
                        <td scope="row"><p>API calls retreiving data from TeamBOT are cached for <strong>1 day</strong>. Please, click clear cache to force a cache refresh.</p></td>
                    </tr>
                    <tr valign="top">
                        <td scope="row"><?php submit_button('Clear cache', 'danger'); ?></td>
                    </tr>
                </table>
            </form>

        </div>
    <?php endif; ?>
</div>